function [W,cp] = LIW_Mode(nmodes,z,N2,omega,f,u0,v0,wavedir)

% Linear Internal Wave Mode solver
% --------------------------------
% Boundary conditions at the surface and bottom are Dirichlet type (mode function equals to zero).
%
% Examples:
%
%   [W,cp] = LIW_Mode(nmodes,z,N2); -- solving with the long wave approximation
%
%   [W,cp] = LIW_Mode(nmodes,z,N2,omega,f); -- finite frequency (or k), including Earth rotation (can be f=0)
%
%   [W,cp] = LIW_Mode(nmodes,z,N2,omega,f,u0,v0,wavedir); -- including both Earth rotation and background shear flow
%
% Input variables: NOTE!! All of the input vectors need to be column vectors.
%                         Also, z is positive pointing up.
%   nmodes -- number of modes to be calculated. If not specified (input an empty array), the first three modes are calculated.
%        z -- depth (m). It has to be equally spaced and includes 0 and D, actual water depth.
%      N2 -- squred buoyancy frequency profile (rad^2/sec^2)
%    omega -- wave frequency (rad/sec)
%        f -- Coriolis freq (rad/s). It can be zero. 
% (u0, v0) -- background flow (m/s)
%  wavedir -- wave direction (rad)
%
% Output:
%      W: mode functions
%     cp: modal phase speeds
%
%     ** Please use k=omega./cp to find wavenumbers, if desired. ***
%
%    By Y.-T. Lin @ WHOI, last edited on 10/18/2012
%
persistent d2dz2 ddz

% checking the input variables
if isempty(nmodes),
    nmodes = 3; end
if ~exist('omega','var')||isempty(omega),
    islongwave = 1; N2(N2==0)=eps; else islongwave = 0;end
if ~exist('u0','var')||isempty(u0),
    isshear = 0; else isshear = 1; end

% depth grids
Nz = length(z); dz = z(2)-z(1);

% create derivative operators
if isempty(d2dz2) || size(d2dz2,2)~=Nz,
    d2dz2 = spdiags([ones(Nz-2,1) -2*ones(Nz-2,1) ones(Nz-2,1)], [0 1 2], Nz-2, Nz);
end
if isshear && (isempty(ddz)||size(ddz,2)~=Nz),
    ddz = spdiags([-.5*ones(Nz-2,1) .5*ones(Nz-2,1)], [0 2], Nz-2, Nz);
end


if ~isshear,
    if islongwave,
        % (1) solve for modes with the long wave approximation
        %     W_zz + N^2/cp^2 W = 0
        A = spdiags( N2(2:end-1), 0, Nz-2, Nz-2);
        B = -d2dz2(:,2:end-1)/dz/dz;
        [W,cpsq] = eigs(A,B,nmodes);
        cp = sqrt(diag(cpsq));
    else
        % (2) solve for modes with Earth rotation
        %     W_zz + (N^2-omega^2)/(omega^2-f^2) k^2 W = 0
        A = d2dz2(:,2:end-1)/dz/dz;
        B = -spdiags( (N2(2:end-1)-omega^2)/(omega^2-f^2), 0, Nz-2, Nz-2);
        [W,ksq] = eigs(A,B,nmodes,'sm');
        k = sqrt(diag(ksq)); cp = omega./k; 
    end
    for ind = 1:nmodes; % normalize each eigenvector to its absolute maximum.
        W(:,ind) = W(:,ind)/max(abs(W(:,ind)));
    end
else
    % (3) solve for modes with rotation and shear flow
    % ------
    % projections of the background flow
    dir = [cos(wavedir) sin(wavedir)]; % wave direction vector
    U = [u0(:) v0(:)]*dir.';
    V = [v0(:) -u0(:)]*dir.';
    
    % forming the coef matrices
    f0 = f/omega;
    
    c5 = (1-f0^2)*d2dz2(:,2:end-1)/dz/dz;
    
    c4 =  spdiags(       d2dz2*U/dz/dz,0,Nz-2,Nz-2) ...
        + spdiags(    -2*f0^2*ddz*U/dz,0,Nz-2,Nz-2)*ddz(:,2:end-1)/dz ...
        + spdiags( (f0^2-3)*U(2:end-1),0,Nz-2,Nz-2)*d2dz2(:,2:end-1)/dz/dz;
    c4 = c4 + 1i*f0* ( spdiags(d2dz2*V/dz/dz,0,Nz-2,Nz-2) ...
        + spdiags(   2*ddz*V/dz,0,Nz-2,Nz-2)*ddz(:,2:end-1)/dz );
    
    c3 =  spdiags(            N2(2:end-1)-omega^2,0,Nz-2,Nz-2) ...
        + spdiags( -2*U(2:end-1).*(d2dz2*U/dz/dz),0,Nz-2,Nz-2) ...
        + spdiags(              3*U(2:end-1).^2,0,Nz-2,Nz-2)*d2dz2(:,2:end-1)/dz/dz;
    c3 = c3 - 1i*f0* ( spdiags(U(2:end-1).*(d2dz2*V/dz/dz),0,Nz-2,Nz-2) ...
        + spdiags(  -2*(ddz*U/dz).*(ddz*V/dz),0,Nz-2,Nz-2) ...
        + spdiags(   2*U(2:end-1).*(ddz*V/dz),0,Nz-2,Nz-2)*ddz(:,2:end-1)/dz );
    
    c2 =  spdiags(      U(2:end-1).^2.*(d2dz2*U/dz/dz),0,Nz-2,Nz-2) ...
        + spdiags( (3*omega^2-N2(2:end-1)).*U(2:end-1),0,Nz-2,Nz-2) ...
        + spdiags(                      -U(2:end-1).^3,0,Nz-2,Nz-2)*d2dz2(:,2:end-1)/dz/dz;
    
    c1 = spdiags( -3*omega^2*U(2:end-1).^2,0,Nz-2,Nz-2);
    
    c0 = spdiags(    omega^2*U(2:end-1).^3,0,Nz-2,Nz-2);
    
    % solve for the polynominal eigenvalues
    %
    % The original Matlab polyeig program does not support sparse matrcies, but
    % the following code utilizes that for faster computation.
    % ------------------------------
    n = Nz-2; p = 5;
    
    % Build two n*p-by-n*p matrices:
    %    A = [A0   0   0 ]   B = [-A1 -A2 -A3 ]
    %        [ 0   I   0 ]       [  I   0   0 ]
    %        [ 0   0   I ]       [  0   I   0 ]
    A = speye(n*p);
    A(1:n,1:n) = -c0;
    B = spdiags(ones(n*(p-1),1),-n,n*p,n*p);
    j = 1:n; B(1:n,j) = c1;
    j = j+n; B(1:n,j) = c2;
    j = j+n; B(1:n,j) = c3;
    j = j+n; B(1:n,j) = c4;
    j = j+n; B(1:n,j) = c5;
    
    % Solve the linearized equation
    [X,cp] = eigs(A,B,nmodes,'lr');
    cp = diag(cp);
    
    % For each eigenvalue, extract the eigenvector from whichever portion
    % of the big eigenvector matrix X gives the smallest normalized residual.
    W = nan(n,nmodes); res = nan(nmodes,1);
    V = nan(n,p);
    for j = 1:nmodes
        V(:) = X(:,j);
        if ~isinf(cp(j)),
            R = c0+cp(j)*c1+cp(j).^2*c2+cp(j).^3*c3+cp(j).^4*c4+cp(j).^5*c5;
        else
            R = c5;
        end
        R = R*V;
        [res(j),ind] = min(sum(abs(R))./ sum(abs(V)));   % Normalized residuals.
        % W(:,j) = V(:,ind)/norm(V(:,ind));  % Eigenvector with unit 2-norm.
        W(:,j) = V(:,ind)/max(abs(V(:,ind)));  % normalize to its maximum.
    end
end

W = [zeros(1,size(W,2)); W; zeros(1,size(W,2))];


return

