% Calculate and inspect internal wave modes in HYCOM output, for verifying
% LIW_Mode.m mode solver.
%
% Duda, T. F., Y.-T. Lin, M. Buijsman and A. E. Newhall, Internal tidal modal ray refraction
% and energy ducting in baroclinic Gulf Stream currents, J. Phys. Oceanogr., 48, 1969-1993,
%  https://doi.org/10.1175/JPO-D-18-0031.1, 2018
%
% Code of YT Lin modified by Tim Duda
% Woods Hole Oceanographic Institution
% Copyright 2018
%
% HYCOM field provided by Maareten Buijsman, U. of Southern Mississippi.
% See above citation for more information
%
% Three user selected parameters: I0, J0, mode type (CPModel.type)
%
% Makes Figures 1,2 and 3
%
%
% Known problems:  Oct 18, 2018:  Can hanfg up and not solvr for TG
% longwave type modes, using certain input U(z),V(z),N() profiles.
%
%




I0 = 66;  % Lat index  range 40-70
J0 =58;   %  Long index  range 40-90


%    Select which of the five cases to examine   --------------------------------
CPModel.type='full' ;     % name derived from "internal tide phase speed model type"
    
    
    % CPModel.type = 'lw-TaylorGoldstein';    % Won't converge , typically
    %                                    
    % CPModel.type = 'finite' 
    % CPModel.type = 'long';
    % CPModel.type = 'TaylorGoldstein'    
    
    %   end selection         ----------------------------
    
    
     
    
    
format compact

%CPModel.Rot=0;
CPModel.Nmodes=3; 
CPModel.dz=10;

CPModel.omega = 2*pi/44700;  % M2 44700
     CPModel.g = 9.8000;

is_reinterp=0;              ind=1;

load Test_Hycom_sub ;

% relocate to grid of smaller subset of the HYCOM field file. 
 
I=I0+1-OceanModel.subgridI(1);
J=J0+1-OceanModel.subgridJ(1);
 
 wd = OceanModel.z(min(find(isnan(OceanModel.Nsq(I,J,:))))-1)
    lat=OceanModel.lat(I,J)
    lon=OceanModel.lon(I,J)
    
      CPModel.Coriolis_frequency = 2*7.2921e-5*sind(lat);
      f=CPModel.Coriolis_frequency;
      omegaoverf= CPModel.omega/f
      
      
      figure(1);clf
       pcolor(OceanModel.lon,OceanModel.lat,OceanModel.bathy); shading flat
       hold on
       plot(OceanModel.lon(I0,J0),OceanModel.lat(I0,J0),'w*')
        plot(OceanModel.lon(I0,J0),OceanModel.lat(I0,J0),'ko')
       [nla,nlo]=size(OceanModel.lat);
       nlat=1:3:nla; nlon=1:3:nlo;
      quiver(OceanModel.lon(nlat,nlon),OceanModel.lat(nlat,nlon),OceanModel.UUpper(nlat,nlon,1),OceanModel.VUpper(nlat,nlon,1),'k');
      colorbar
      xlabel('degrees'); ylabel('degrees')
      set(gca,'dataaspectratio',[ 1 cosd(mean(OceanModel.lat(:))) 1])
      title ('Hycom output domain, lat/long')
      
      drawnow
      
       figure(2);clf
       pcolor(OceanModel.bathy); shading flat
        hold on
           plot(J0,I0,'w*')
           plot(J0,I0,'ko')
        [nla,nlo]=size(OceanModel.lat);
        nlat=1:3:nla; nlon=1:3:nlo;
       quiver(nlon,nlat,OceanModel.UUpper(nlat,nlon,1),OceanModel.VUpper(nlat,nlon,1),'k');
       colorbar 
       title ('Hycom output domain, by index')
       xlabel('index J'); ylabel('index I')
       axis equal 
       title('area with full-depth profiles outlined')
       plot([ OceanModel.subgridJ(1) OceanModel.subgridJ(1) OceanModel.subgridJ(end)  OceanModel.subgridJ(end) OceanModel.subgridJ(1)], ...
         [ OceanModel.subgridI(1) OceanModel.subgridI(end) OceanModel.subgridI(end)  OceanModel.subgridI(1) OceanModel.subgridI(1)],'w');
       drawnow 
       
       
    if is_reinterp,
        N2_tmp = squeeze(OceanModel.Nsq(I,J,:)).';
        z_tmp = OceanModel.z; z_tmp = z_tmp(:).';
        [z_tmp, N2_tmp]=fixnsq2(wd,N2_tmp,z_tmp);
        z = linspace(0,wd,round(abs(wd)/CPModel.dz)+1); z = z(:);  % the LIW_Mode program need evenly spaced grids
        if (length(z) > 250) && (strcmpi(CPModel.type(1:2),'ta')||strcmpi(CPModel.type(1:2),'fu')||strcmpi(CPModel.type(1:4),'lw-t')),
            z = linspace(z(1),z(end),250); z = z(:);
        end
        N2 = interp1(z_tmp, N2_tmp, z,'linear','extrap');
        uin = interp1(OceanModel.z(:),squeeze(OceanModel.U(I,J,:)).', z,'linear','extrap'); uin(isnan(uin)) = 0;
        vin = interp1(OceanModel.z(:),squeeze(OceanModel.V(I,J,:)).', z,'linear','extrap'); vin(isnan(vin)) = 0;
        %[u0,v0]=cart_rotation(uin,vin,CPModel.Rot);    % rotate into xy ray-trace coord system, by .Rot degrees TFD
    else
        z = OceanModel.z(OceanModel.z>=wd); z = z(:);     
        N2 = squeeze(OceanModel.Nsq(I,J,OceanModel.z>=wd));
        CFreq = 2*7.2921e-5*sind(OceanModel.lat(I,J));
        
       %  uin = squeeze(OceanModel.U(I,J,OceanModel.z>=wd))/100;      % FOR mseas only
       %  vin = squeeze(OceanModel.V(I,J,OceanModel.z>=wd))/100;      % FOR mseas only
        
        uin = squeeze(OceanModel.U(I,J,OceanModel.z>=wd));             % NEEDS M/S
        vin = squeeze(OceanModel.V(I,J,OceanModel.z>=wd)); 
        
        z = abs(z);    % z has to be positive for LIW_Mode function
        
       % [u0,v0]=cart_rotation(uin,vin,CPModel.Rot);    % rotate into xy ray-trace coord system, by .Rot degrees TFD
    end
    
    
    
    wavedir= pi*[-1:.02:1];
    
       cp1=[]; cp2=[]; w1=[]; w2=[];
       
    if strcmpi(CPModel.type(1:4),'long')
        for ith = 1:length(wavedir)
            [W,cp] = LIW_Mode(CPModel.Nmodes,z,N2);
                 w1=[w1 W(:,1)];
         %    w2=[w2 W(:,2)];
            cp1=[cp1 real(cp(1))];
            cp2=[cp2 real(cp(2))];      
        end
        
    
    elseif strcmpi(CPModel.type(1:4),'fini')
           for ith = 1:length(wavedir)
         %[W,cp] = LIW_Mode(CPModel.Nmodes,z,N2,CPModel.omega,CFreq);  
          [W,cp] = LIW_Mode(CPModel.Nmodes,z,N2,CPModel.omega,CPModel.Coriolis_frequency);
      
        w1=[w1 W(:,1)];
         %    w2=[w2 W(:,2)];
            cp1=[cp1 real(cp(1))];
            cp2=[cp2 real(cp(2))];
           end
            
    elseif strcmpi(CPModel.type(1:4),'lw-t'),
        %ind
        for ith = 1:length(wavedir)
            ith
            [W,cp] = LIW_Mode_2(CPModel.Nmodes,z,N2,0,0,uin,vin,wavedir(ith));
             w1=[w1 W(:,1)];
         %    w2=[w2 W(:,2)];
            cp1=[cp1 real(cp(1))];
            cp2=[cp2 real(cp(2))];      
        end
        
    elseif strcmpi(CPModel.type(1:4),'tayl'),
        for ith = 1:length(wavedir)
            [W,cp] = LIW_Mode(CPModel.Nmodes,z,N2,CPModel.omega,0,uin,vin,wavedir(ith));
       %    
         w1=[w1 W(:,1)];
         %    w2=[w2 W(:,2)];
            cp1=[cp1 real(cp(1))];
            cp2=[cp2 real(cp(2))];
          
        end
    elseif strcmpi(CPModel.type(1:4),'full'),
        % par
        cp1=[]; cp2=[]; w1=[]; w2=[];
        %ubt=mean(uin)*ones(size(uin));
        %vbt=mean(vin)*ones(size(vin));
        for ith = 1:length(wavedir)
            % [W,cp] = LIW_Mode(CPModel.Nmodes,z,N2,CPModel.omega,CPModel.Coriolis_frequency,u0,v0,wavedir(ith));
            [W,cp] = LIW_Mode(CPModel.Nmodes,z,N2,CPModel.omega,CPModel.Coriolis_frequency,uin,vin,wavedir(ith));  % UIN V IN used
        
            w1=[w1 W(:,1)];
         %    w2=[w2 W(:,2)];
            cp1=[cp1 real(cp(1))];
            cp2=[cp2 real(cp(2))];
            
        end
        
    end
        
 %%
        figure(3);clf
        
        subplot(221)  % cp
%  polarplot(wavedir, cp1,'k' ,'linewidth',2)  % for newer matlab releases such as 2017

hhh=polar(wavedir, cp1);
set(hhh,'color','k' ,'linewidth',2)

hold on

%  polarplot(wavedir, cp2,'k--' ,'linewidth',2 )
hhh=polar(wavedir, cp2);
set(hhh,'color','k' ,'linewidth',2,'linestyle','--')

 
  %plot(uin*3,vin*3)
  % polarplot(angle(uin+1i*vin),abs(uin+1i*vin),'k') 
  
  hhh=polar( angle(uin+1i*vin),abs(uin+1i*vin));
  set(hhh,'color','k' ,'linestyle','--')
  
  
  title(['(a) Modes 1 and 2 speed vs direction, scaled current  ' ],'fontweight' ,'normal')
  % axis( [ -10 10 -10 10 ])
  
  subplot(234)  % u
  plot(uin,z,'k','linewidth',2)
  hold on
  plot(vin,z,'k--','linewidth' ,2)
  axis ij
  set(gca,'ylim',[0 -wd])
  xlabel('u,v(dash)  (m/s)')
  ylabel('z (m)')
  grid on
  title('(c)       ','fontweight' ,'normal')
  
  subplot(222)  % W shapes
   % xx=squeeze(real(OM_W(nn,:,:,1))).';
   xx=w1;
   xm=max(abs(xx),[],1);
   xmm=ones(size(xx,1),1)*xm;
   x2=abs(xx)./xmm;
   %plot(xx)
   pcolor(wavedir*180/pi, z, x2); shading flat
   li=find( isnan(xx(:,1))==1,1);
   if ~isempty(li)
  set(gca,'ylim', [0 li] )  
   end
   xlabel('direction, deg.')
  ylabel('depth (m)')
   axis ij
   colorbar
   title('(b)  Mode shapes     ','fontweight' ,'normal')
   
   subplot(236)
  surf( abs(x2).*(ones(size(xx,1),1)*cos(wavedir))  ,...
       abs(x2).*(ones(size(xx,1),1)*sin(wavedir))  , abs(wd)/length(N2)*-1*[1:length(N2)].'*ones(1, length(wavedir)), abs(x2) ); 
   shading flat
   % plot(abs(xx))
   hold on
   plot3(.5*uin, .5*vin, -5000*ones(size(uin)))
     set(gca,'zlim', [wd 0] )  
    if ~isempty(li)
  set(gca,'zlim', [wd 0] )  
  caxis( [-li 0])
    end

   title(['(e) Modes, location ' num2str( lat) ' N, ' num2str(360-lon) 'W' ],'fontweight' ,'normal')
 
  subplot(235)
  plot(sqrt(N2)* 3600/(2*pi) , z,'k','linewidth',2 )
    set(gca,'ylim', [ 0 -wd] )
    xlabel('N (cph)')
    grid on
    axis ij
    title('(d)         ','fontweight' ,'normal')
    
    drawnow
    
    